@@ -267,12 +267,65 @@ class TestClientStorageURIResolver(unittest.TestCase):
         storage.close()
         ClientStorage.assert_called_once_with('/var/nosuchfile', wait=0)
 
+    @mock.patch('zodburi.resolvers.ClientStorage')
+    def test_factory_kwargs(self, ClientStorage):
+        resolver = self._makeOne()
+        factory, dbkw = resolver('zeo:///var/nosuchfile?'
+                                 'storage=main&'
+                                 'cache_size=1kb&'
+                                 'name=foo&'
+                                 'client=bar&'
+                                 'var=baz&'
+                                 'min_disconnect_poll=2&'
+                                 'max_disconnect_poll=3&'
+                                 'wait_for_server_on_startup=true&'
+                                 'wait=4&'
+                                 'wait_timeout=5&'
+                                 'read_only=6&'
+                                 'read_only_fallback=7&'
+                                 'drop_cache_rather_verify=true&'
+                                 'username=monty&'
+                                 'password=python&'
+                                 'realm=blat&'
+                                 'blob_dir=some/dir&'
+                                 'shared_blob_dir=true&'
+                                 'blob_cache_size=1kb&'
+                                 'blob_cache_size_check=8&'
+                                 'client_label=fink&'
+                                 )
+        storage = factory()
+        storage.close()
+        ClientStorage.assert_called_once_with('/var/nosuchfile',
+                                              storage='main',
+                                              cache_size=1024,
+                                              name='foo',
+                                              client='bar',
+                                              var='baz',
+                                              min_disconnect_poll=2,
+                                              max_disconnect_poll=3,
+                                              wait_for_server_on_startup=1,
+                                              wait=4,
+                                              wait_timeout=5,
+                                              read_only=6,
+                                              read_only_fallback=7,
+                                              drop_cache_rather_verify=1,
+                                              username='monty',
+                                              password='python',
+                                              realm='blat',
+                                              blob_dir='some/dir',
+                                              shared_blob_dir=1,
+                                              blob_cache_size=1024,
+                                              blob_cache_size_check=8,
+                                              client_label='fink',
+                                              )
+
+
     @mock.patch('zodburi.resolvers.ClientStorage')
     def test_invoke_factory_demostorage(self, ClientStorage):
         from ZODB.DemoStorage import DemoStorage
         resolver = self._makeOne()
         factory, dbkw = resolver('zeo:///var/nosuchfile?wait=false'
-                                        '&demostorage=true')
+                                 '&demostorage=true')
         storage = factory()
         storage.close()
         self.assertTrue(isinstance(storage, DemoStorage))
@@ -280,9 +333,9 @@ class TestClientStorageURIResolver(unittest.TestCase):
     def test_dbargs(self):
         resolver = self._makeOne()
         factory, dbkw = resolver('zeo://localhost:8080?debug=true&'
-                                        'connection_pool_size=1&'
-                                        'connection_cache_size=1&'
-                                        'database_name=dbname')
+                                 'connection_pool_size=1&'
+                                 'connection_cache_size=1&'
+                                 'database_name=dbname')
         self.assertEqual(dbkw, {'connection_pool_size': '1',
                                 'connection_cache_size': '1',
                                 'database_name': 'dbname'})
@@ -370,9 +423,15 @@ class TestZConfigURIResolver(unittest.TestCase):
         storage = factory()
         from ZODB.MappingStorage import MappingStorage
         self.assertTrue(isinstance(storage, MappingStorage))
-        self.assertEqual(dbkw, {
-            'connection_cache_size': 5000,
-            'connection_pool_size': 7})
+        self.assertEqual(dbkw,
+                         {'connection_cache_size': 5000,
+                          'connection_cache_size_bytes': 0,
+                          'connection_historical_cache_size': 1000,
+                          'connection_historical_cache_size_bytes': 0,
+                          'connection_historical_pool_size': 3,
+                          'connection_historical_timeout': 300,
+                          'connection_large_record_size': 16777216,
+                          'connection_pool_size': 7})
 
     def test_named_database(self):
         self.tmp.write(b"""
@@ -390,10 +449,71 @@ class TestZConfigURIResolver(unittest.TestCase):
         storage = factory()
         from ZODB.MappingStorage import MappingStorage
         self.assertTrue(isinstance(storage, MappingStorage))
-        self.assertEqual(dbkw, {
-            'connection_cache_size': 20000,
-            'connection_pool_size': 5,
-            'database_name': 'foo'})
+        self.assertEqual(dbkw,
+                         {'connection_cache_size': 20000,
+                          'connection_cache_size_bytes': 0,
+                          'connection_historical_cache_size': 1000,
+                          'connection_historical_cache_size_bytes': 0,
+                          'connection_historical_pool_size': 3,
+                          'connection_historical_timeout': 300,
+                          'connection_large_record_size': 16777216,
+                          'connection_pool_size': 5,
+                          'database_name': 'foo'})
+
+    def test_database_all_options(self):
+        from zodburi import connection_parameters, bytes_parameters
+        self.tmp.write(("""
+        <zodb x>
+          <mappingstorage>
+          </mappingstorage>
+          database-name foo
+          %s
+        </zodb>
+        """ % '\n'.join("%s %s" % (
+                            name.replace('_', '-'),
+                            '%sMB' % i if name in bytes_parameters else i,
+                           )
+                        for (i, name)
+                        in enumerate(connection_parameters)
+                        )).encode())
+        self.tmp.flush()
+        resolver = self._makeOne()
+        factory, dbkw = resolver('zconfig://%s#x' % self.tmp.name)
+        storage = factory()
+        from ZODB.MappingStorage import MappingStorage
+        self.assertTrue(isinstance(storage, MappingStorage))
+        expect = dict(database_name='foo')
+        for i, parameter in enumerate(connection_parameters):
+            cparameter = 'connection_' + parameter
+            expect[cparameter] = i
+            if parameter in bytes_parameters:
+                expect[cparameter] *= 1<<20
+        self.assertEqual(dbkw, expect)
+
+    def test_database_integration_because_ints(self):
+        from zodburi import resolve_uri
+        self.tmp.write(b"""
+        <zodb>
+          <mappingstorage>
+          </mappingstorage>
+        </zodb>
+        """)
+        self.tmp.flush()
+        from zodburi import resolve_uri
+        factory, dbkw = resolve_uri('zconfig://%s' % self.tmp.name)
+        storage = factory()
+        from ZODB.MappingStorage import MappingStorage
+        self.assertTrue(isinstance(storage, MappingStorage))
+        self.assertEqual(dbkw,
+                         {'cache_size': 5000,
+                          'cache_size_bytes': 0,
+                          'historical_cache_size': 1000,
+                          'historical_cache_size_bytes': 0,
+                          'historical_pool_size': 3,
+                          'historical_timeout': 300,
+                          'large_record_size': 16777216,
+                          'pool_size': 7,
+                          'database_name': 'unnamed'})
 
 class TestMappingStorageURIResolver(Base, unittest.TestCase):
 
